@@ -5,27 +5,35 @@ import (
 	"encoding/json"
 	"errors"
 	"fmt"
+	"io/ioutil"
+	"strings"
 	"testing"
 	"time"
 
-	"github.com/ghodss/yaml"
 	"github.com/sirupsen/logrus"
 	"github.com/stretchr/testify/require"
 	"golang.org/x/time/rate"
+	"gopkg.in/yaml.v2"
 	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
 	rbacv1 "k8s.io/api/rbac/v1"
 	"k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1"
 	apiextensionsfake "k8s.io/apiextensions-apiserver/pkg/client/clientset/clientset/fake"
+	"k8s.io/apimachinery/pkg/api/meta"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
 	"k8s.io/apimachinery/pkg/runtime"
+	"k8s.io/apimachinery/pkg/runtime/schema"
 	"k8s.io/apimachinery/pkg/types"
 	utilclock "k8s.io/apimachinery/pkg/util/clock"
+	utilyaml "k8s.io/apimachinery/pkg/util/yaml"
+	fakedynamic "k8s.io/client-go/dynamic/fake"
 	"k8s.io/client-go/informers"
 	k8sfake "k8s.io/client-go/kubernetes/fake"
 	"k8s.io/client-go/rest"
 	"k8s.io/client-go/tools/cache"
 	"k8s.io/client-go/util/workqueue"
+	apiregistrationv1 "k8s.io/kube-aggregator/pkg/apis/apiregistration/v1"
 	apiregistrationfake "k8s.io/kube-aggregator/pkg/client/clientset_generated/clientset/fake"
 
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/api/apis/operators/v1alpha1"
@@ -340,6 +348,7 @@ func TestExecutePlan(t *testing.T) {
 	tests := []struct {
 		testName string
 		in       *v1alpha1.InstallPlan
+		extObjs  []runtime.Object
 		want     []runtime.Object
 		err      error
 	}{
@@ -361,7 +370,7 @@ func TestExecutePlan(t *testing.T) {
 							Version:                "v1",
 							Kind:                   "Service",
 							Name:                   "service",
-							Manifest:               toManifest(service("service", namespace)),
+							Manifest:               toManifest(t, service("service", namespace)),
 						},
 						Status: v1alpha1.StepStatusUnknown,
 					},
@@ -373,7 +382,7 @@ func TestExecutePlan(t *testing.T) {
 							Version:                "v1alpha1",
 							Kind:                   "ClusterServiceVersion",
 							Name:                   "csv",
-							Manifest:               toManifest(csv("csv", namespace, nil, nil)),
+							Manifest:               toManifest(t, csv("csv", namespace, nil, nil)),
 						},
 						Status: v1alpha1.StepStatusUnknown,
 					},
@@ -394,7 +403,7 @@ func TestExecutePlan(t *testing.T) {
 							Version:                "v1",
 							Kind:                   "ServiceAccount",
 							Name:                   "sa",
-							Manifest: toManifest(serviceAccount("sa", namespace, "",
+							Manifest: toManifest(t, serviceAccount("sa", namespace, "",
 								objectReference("init secret"))),
 						},
 						Status: v1alpha1.StepStatusUnknown,
@@ -416,7 +425,7 @@ func TestExecutePlan(t *testing.T) {
 							Version:                "v1",
 							Kind:                   "ServiceAccount",
 							Name:                   "sa",
-							Manifest: toManifest(serviceAccount("sa", namespace, "name",
+							Manifest: toManifest(t, serviceAccount("sa", namespace, "name",
 								objectReference("init secret"))),
 						},
 						Status: v1alpha1.StepStatusUnknown,
@@ -429,7 +438,7 @@ func TestExecutePlan(t *testing.T) {
 							Version:                "v1",
 							Kind:                   "ServiceAccount",
 							Name:                   "sa",
-							Manifest:               toManifest(serviceAccount("sa", namespace, "name", nil)),
+							Manifest:               toManifest(t, serviceAccount("sa", namespace, "name", nil)),
 						},
 						Status: v1alpha1.StepStatusUnknown,
 					},
@@ -450,7 +459,7 @@ func TestExecutePlan(t *testing.T) {
 							Version:                "v1",
 							Kind:                   "ServiceAccount",
 							Name:                   "sa",
-							Manifest: toManifest(serviceAccount("sa", namespace, "old_name",
+							Manifest: toManifest(t, serviceAccount("sa", namespace, "old_name",
 								objectReference("init secret"))),
 						},
 						Status: v1alpha1.StepStatusUnknown,
@@ -463,7 +472,7 @@ func TestExecutePlan(t *testing.T) {
 							Version:                "v1",
 							Kind:                   "ServiceAccount",
 							Name:                   "sa",
-							Manifest:               toManifest(serviceAccount("sa", namespace, "new_name", nil)),
+							Manifest:               toManifest(t, serviceAccount("sa", namespace, "new_name", nil)),
 						},
 						Status: v1alpha1.StepStatusUnknown,
 					},
@@ -472,6 +481,48 @@ func TestExecutePlan(t *testing.T) {
 			want: []runtime.Object{serviceAccount("sa", namespace, "new_name", objectReference("init secret"))},
 			err:  nil,
 		},
+		{
+			testName: "DynamicResourcesAreOwnerReferencedToCSV",
+			in: withSteps(installPlan("p", namespace, v1alpha1.InstallPlanPhaseInstalling, "csv"),
+				[]*v1alpha1.Step{
+					{
+						Resolving: "csv",
+						Resource: v1alpha1.StepResource{
+							CatalogSource:          "catalog",
+							CatalogSourceNamespace: namespace,
+							Group:                  "operators.coreos.com",
+							Version:                "v1alpha1",
+							Kind:                   "ClusterServiceVersion",
+							Name:                   "csv",
+							Manifest:               toManifest(t, csv("csv", namespace, nil, nil)),
+						},
+						Status: v1alpha1.StepStatusUnknown,
+					},
+					{
+						Resolving: "csv",
+						Resource: v1alpha1.StepResource{
+							CatalogSource:          "catalog",
+							CatalogSourceNamespace: namespace,
+							Group:                  "monitoring.coreos.com",
+							Version:                "v1",
+							Kind:                   "PrometheusRule",
+							Name:                   "rule",
+							Manifest:               toManifest(t, decodeFile(t, "./testdata/prometheusrule.cr.yaml", &unstructured.Unstructured{})),
+						},
+						Status: v1alpha1.StepStatusUnknown,
+					},
+				},
+			),
+			extObjs: []runtime.Object{decodeFile(t, "./testdata/prometheusrule.crd.yaml", &v1beta1.CustomResourceDefinition{})},
+			want: []runtime.Object{
+				csv("csv", namespace, nil, nil),
+				modify(t, decodeFile(t, "./testdata/prometheusrule.cr.yaml", &unstructured.Unstructured{}),
+					withNamespace(namespace),
+					withOwner(csv("csv", namespace, nil, nil)),
+				),
+			},
+			err: nil,
+		},
 	}
 
 	for _, tt := range tests {
@@ -479,12 +530,13 @@ func TestExecutePlan(t *testing.T) {
 			ctx, cancel := context.WithCancel(context.TODO())
 			defer cancel()
 
-			op, err := NewFakeOperator(ctx, namespace, []string{namespace}, withClientObjs(tt.in))
+			op, err := NewFakeOperator(ctx, namespace, []string{namespace}, withClientObjs(tt.in), withExtObjs(tt.extObjs...))
 			require.NoError(t, err)
 
 			err = op.ExecutePlan(tt.in)
 			require.Equal(t, tt.err, err)
 
+			getOpts := metav1.GetOptions{}
 			for _, obj := range tt.want {
 				var err error
 				var fetched runtime.Object
@@ -505,8 +557,29 @@ func TestExecutePlan(t *testing.T) {
 					fetched, err = op.opClient.GetSecret(namespace, o.GetName())
 				case *corev1.Service:
 					fetched, err = op.opClient.GetService(namespace, o.GetName())
+				case *v1beta1.CustomResourceDefinition:
+					fetched, err = op.opClient.ApiextensionsV1beta1Interface().ApiextensionsV1beta1().CustomResourceDefinitions().Get(o.GetName(), getOpts)
 				case *v1alpha1.ClusterServiceVersion:
-					fetched, err = op.client.OperatorsV1alpha1().ClusterServiceVersions(namespace).Get(o.GetName(), metav1.GetOptions{})
+					fetched, err = op.client.OperatorsV1alpha1().ClusterServiceVersions(namespace).Get(o.GetName(), getOpts)
+				case *unstructured.Unstructured:
+					// Get the resource from the GVK
+					gvk := o.GroupVersionKind()
+					var r metav1.APIResource
+					r, err = op.apiresourceFromGVK(gvk)
+					require.NoError(t, err)
+
+					gvr := schema.GroupVersionResource{
+						Group:    gvk.Group,
+						Version:  gvk.Version,
+						Resource: r.Name,
+					}
+
+					if r.Namespaced {
+						fetched, err = op.dynamicClient.Resource(gvr).Namespace(namespace).Get(o.GetName(), getOpts)
+						break
+					}
+
+					fetched, err = op.dynamicClient.Resource(gvr).Get(o.GetName(), getOpts)
 				default:
 					require.Failf(t, "couldn't find expected object", "%#v", obj)
 				}
@@ -518,6 +591,87 @@ func TestExecutePlan(t *testing.T) {
 	}
 }
 
+func TestSupportedDynamicResources(t *testing.T) {
+	tests := []struct {
+		testName       string
+		resource       v1alpha1.StepResource
+		expectedResult bool
+	}{
+		{
+			testName: "UnsupportedObject",
+			resource: v1alpha1.StepResource{
+				Kind: "UnsupportedKind",
+			},
+			expectedResult: false,
+		},
+		{
+			testName: "ServiceMonitorResource",
+			resource: v1alpha1.StepResource{
+				Kind: "ServiceMonitor",
+			},
+			expectedResult: true,
+		},
+		{
+			testName: "UnsupportedObject",
+			resource: v1alpha1.StepResource{
+				Kind: "PrometheusRule",
+			},
+			expectedResult: true,
+		},
+	}
+
+	for _, tt := range tests {
+		t.Run(tt.testName, func(t *testing.T) {
+			require.Equal(t, tt.expectedResult, isSupported(tt.resource.Kind))
+		})
+	}
+}
+
+func TestExecutePlanDynamicResources(t *testing.T) {
+	namespace := "ns"
+	unsupportedYaml := yamlFromFilePath(t, "testdata/unsupportedkind.cr.yaml")
+
+	tests := []struct {
+		testName string
+		in       *v1alpha1.InstallPlan
+		err      error
+	}{
+		{
+			testName: "UnsupportedObject",
+			in: withSteps(installPlan("p", namespace, v1alpha1.InstallPlanPhaseInstalling, "csv"),
+				[]*v1alpha1.Step{
+					{
+						Resource: v1alpha1.StepResource{
+							CatalogSource:          "catalog",
+							CatalogSourceNamespace: namespace,
+							Group:                  "some.unsupported.group",
+							Version:                "v1",
+							Kind:                   "UnsupportedKind",
+							Name:                   "unsupportedkind",
+							Manifest:               unsupportedYaml,
+						},
+						Status: v1alpha1.StepStatusUnknown,
+					},
+				},
+			),
+			err: v1alpha1.ErrInvalidInstallPlan,
+		},
+	}
+
+	for _, tt := range tests {
+		t.Run(tt.testName, func(t *testing.T) {
+			ctx, cancel := context.WithCancel(context.TODO())
+			defer cancel()
+
+			op, err := NewFakeOperator(ctx, namespace, []string{namespace}, withClientObjs(tt.in))
+			require.NoError(t, err)
+
+			err = op.ExecutePlan(tt.in)
+			require.Equal(t, tt.err, err)
+		})
+	}
+}
+
 func TestSyncCatalogSources(t *testing.T) {
 	clockFake := utilclock.NewFakeClock(time.Date(2018, time.January, 26, 20, 40, 0, 0, time.UTC))
 	now := metav1.NewTime(clockFake.Now())
@@ -594,7 +748,7 @@ func TestSyncCatalogSources(t *testing.T) {
 					Namespace:       "cool-namespace",
 					UID:             types.UID("configmap-uid"),
 					ResourceVersion: "resource-version",
-					LastUpdateTime: now,
+					LastUpdateTime:  now,
 				},
 				RegistryServiceStatus: nil,
 			},
@@ -957,7 +1111,7 @@ func withK8sObjs(k8sObjs ...runtime.Object) fakeOperatorOption {
 	}
 }
 
-func extObjs(extObjs ...runtime.Object) fakeOperatorOption {
+func withExtObjs(extObjs ...runtime.Object) fakeOperatorOption {
 	return func(config *fakeOperatorConfig) {
 		config.extObjs = extObjs
 	}
@@ -970,7 +1124,7 @@ func withFakeClientOptions(options ...clientfake.Option) fakeOperatorOption {
 }
 
 // NewFakeOperator creates a new operator using fake clients.
-func NewFakeOperator(ctx context.Context, namespace string, watchedNamespaces []string, fakeOptions ...fakeOperatorOption) (*Operator, error) {
+func NewFakeOperator(ctx context.Context, namespace string, namespaces []string, fakeOptions ...fakeOperatorOption) (*Operator, error) {
 	// Apply options to default config
 	config := &fakeOperatorConfig{
 		logger:   logrus.StandardLogger(),
@@ -983,7 +1137,12 @@ func NewFakeOperator(ctx context.Context, namespace string, watchedNamespaces []
 
 	// Create client fakes
 	clientFake := fake.NewReactionForwardingClientsetDecorator(config.clientObjs, config.clientOptions...)
-	opClientFake := operatorclient.NewClient(k8sfake.NewSimpleClientset(config.k8sObjs...), apiextensionsfake.NewSimpleClientset(config.extObjs...), apiregistrationfake.NewSimpleClientset(config.regObjs...))
+	// TODO: Using the ReactionForwardingClientsetDecorator for k8s objects causes issues with adding Resources for discovery.
+	// For now, directly use a SimpleClientset instead.
+	k8sClientFake := k8sfake.NewSimpleClientset(config.k8sObjs...)
+	k8sClientFake.Resources = apiResourcesForObjects(append(config.extObjs, config.regObjs...))
+	opClientFake := operatorclient.NewClient(k8sClientFake, apiextensionsfake.NewSimpleClientset(config.extObjs...), apiregistrationfake.NewSimpleClientset(config.regObjs...))
+	dynamicClientFake := fakedynamic.NewSimpleDynamicClient(runtime.NewScheme())
 
 	// Create operator namespace
 	_, err := opClientFake.KubernetesInterface().CoreV1().Namespaces().Create(&corev1.Namespace{ObjectMeta: metav1.ObjectMeta{Name: namespace}})
@@ -994,45 +1153,43 @@ func NewFakeOperator(ctx context.Context, namespace string, watchedNamespaces []
 	wakeupInterval := 5 * time.Minute
 	lister := operatorlister.NewLister()
 	var sharedInformers []cache.SharedIndexInformer
-	for _, ns := range watchedNamespaces {
+	for _, ns := range namespaces {
 		if ns != namespace {
-			_, err := opClientFake.KubernetesInterface().CoreV1().Namespaces().Create(&corev1.Namespace{ObjectMeta: metav1.ObjectMeta{Name: namespace}})
+			_, err := opClientFake.KubernetesInterface().CoreV1().Namespaces().Create(&corev1.Namespace{ObjectMeta: metav1.ObjectMeta{Name: ns}})
 			if err != nil {
 				return nil, err
 			}
 		}
-
-		// Create informers and register listers
-		operatorsFactory := externalversions.NewSharedInformerFactoryWithOptions(clientFake, wakeupInterval, externalversions.WithNamespace(ns))
-		catsrcInformer := operatorsFactory.Operators().V1alpha1().CatalogSources()
-		subInformer := operatorsFactory.Operators().V1alpha1().Subscriptions()
-		ipInformer := operatorsFactory.Operators().V1alpha1().InstallPlans()
-		csvInformer := operatorsFactory.Operators().V1alpha1().ClusterServiceVersions()
-		sharedInformers = append(sharedInformers, catsrcInformer.Informer(), subInformer.Informer(), ipInformer.Informer(), csvInformer.Informer())
-
-		lister.OperatorsV1alpha1().RegisterCatalogSourceLister(ns, catsrcInformer.Lister())
-		lister.OperatorsV1alpha1().RegisterSubscriptionLister(ns, subInformer.Lister())
-		lister.OperatorsV1alpha1().RegisterInstallPlanLister(ns, ipInformer.Lister())
-		lister.OperatorsV1alpha1().RegisterClusterServiceVersionLister(ns, csvInformer.Lister())
-
-		factory := informers.NewSharedInformerFactoryWithOptions(opClientFake.KubernetesInterface(), wakeupInterval, informers.WithNamespace(ns))
-		roleInformer := factory.Rbac().V1().Roles()
-		roleBindingInformer := factory.Rbac().V1().RoleBindings()
-		serviceAccountInformer := factory.Core().V1().ServiceAccounts()
-		serviceInformer := factory.Core().V1().Services()
-		podInformer := factory.Core().V1().Pods()
-		configMapInformer := factory.Core().V1().ConfigMaps()
-		sharedInformers = append(sharedInformers, roleInformer.Informer(), roleBindingInformer.Informer(), serviceAccountInformer.Informer(), serviceInformer.Informer(), podInformer.Informer(), configMapInformer.Informer())
-
-		lister.RbacV1().RegisterRoleLister(ns, roleInformer.Lister())
-		lister.RbacV1().RegisterRoleBindingLister(ns, roleBindingInformer.Lister())
-		lister.CoreV1().RegisterServiceAccountLister(ns, serviceAccountInformer.Lister())
-		lister.CoreV1().RegisterServiceLister(ns, serviceInformer.Lister())
-		lister.CoreV1().RegisterPodLister(ns, podInformer.Lister())
-		lister.CoreV1().RegisterConfigMapLister(ns, configMapInformer.Lister())
-
 	}
 
+	// Create informers and register listers
+	operatorsFactory := externalversions.NewSharedInformerFactoryWithOptions(clientFake, wakeupInterval, externalversions.WithNamespace(metav1.NamespaceAll))
+	catsrcInformer := operatorsFactory.Operators().V1alpha1().CatalogSources()
+	subInformer := operatorsFactory.Operators().V1alpha1().Subscriptions()
+	ipInformer := operatorsFactory.Operators().V1alpha1().InstallPlans()
+	csvInformer := operatorsFactory.Operators().V1alpha1().ClusterServiceVersions()
+	sharedInformers = append(sharedInformers, catsrcInformer.Informer(), subInformer.Informer(), ipInformer.Informer(), csvInformer.Informer())
+
+	lister.OperatorsV1alpha1().RegisterCatalogSourceLister(metav1.NamespaceAll, catsrcInformer.Lister())
+	lister.OperatorsV1alpha1().RegisterSubscriptionLister(metav1.NamespaceAll, subInformer.Lister())
+	lister.OperatorsV1alpha1().RegisterInstallPlanLister(metav1.NamespaceAll, ipInformer.Lister())
+	lister.OperatorsV1alpha1().RegisterClusterServiceVersionLister(metav1.NamespaceAll, csvInformer.Lister())
+
+	factory := informers.NewSharedInformerFactoryWithOptions(opClientFake.KubernetesInterface(), wakeupInterval, informers.WithNamespace(metav1.NamespaceAll))
+	roleInformer := factory.Rbac().V1().Roles()
+	roleBindingInformer := factory.Rbac().V1().RoleBindings()
+	serviceAccountInformer := factory.Core().V1().ServiceAccounts()
+	serviceInformer := factory.Core().V1().Services()
+	podInformer := factory.Core().V1().Pods()
+	configMapInformer := factory.Core().V1().ConfigMaps()
+	sharedInformers = append(sharedInformers, roleInformer.Informer(), roleBindingInformer.Informer(), serviceAccountInformer.Informer(), serviceInformer.Informer(), podInformer.Informer(), configMapInformer.Informer())
+
+	lister.RbacV1().RegisterRoleLister(metav1.NamespaceAll, roleInformer.Lister())
+	lister.RbacV1().RegisterRoleBindingLister(metav1.NamespaceAll, roleBindingInformer.Lister())
+	lister.CoreV1().RegisterServiceAccountLister(metav1.NamespaceAll, serviceAccountInformer.Lister())
+	lister.CoreV1().RegisterServiceLister(metav1.NamespaceAll, serviceInformer.Lister())
+	lister.CoreV1().RegisterPodLister(metav1.NamespaceAll, podInformer.Lister())
+	lister.CoreV1().RegisterConfigMapLister(metav1.NamespaceAll, configMapInformer.Lister())
 	logger := logrus.New()
 
 	// Create the new operator
@@ -1042,13 +1199,14 @@ func NewFakeOperator(ctx context.Context, namespace string, watchedNamespaces []
 	}
 
 	op := &Operator{
-		Operator:  queueOperator,
-		clock:     config.clock,
-		logger:    config.logger,
-		opClient:  opClientFake,
-		client:    clientFake,
-		lister:    lister,
-		namespace: namespace,
+		Operator:      queueOperator,
+		clock:         config.clock,
+		logger:        config.logger,
+		opClient:      opClientFake,
+		dynamicClient: dynamicClientFake,
+		client:        clientFake,
+		lister:        lister,
+		namespace:     namespace,
 		nsResolveQueue: workqueue.NewNamedRateLimitingQueue(
 			workqueue.NewMaxOfRateLimiter(
 				workqueue.NewItemExponentialFailureRateLimiter(1*time.Second, 1000*time.Second),
@@ -1057,9 +1215,9 @@ func NewFakeOperator(ctx context.Context, namespace string, watchedNamespaces []
 			), "resolver"),
 		resolver:              config.resolver,
 		reconciler:            config.reconciler,
-		clientAttenuator:      scoped.NewClientAttenuator(logger, &rest.Config{}, opClientFake, clientFake),
+		clientAttenuator:      scoped.NewClientAttenuator(logger, &rest.Config{}, opClientFake, clientFake, dynamicClientFake),
 		serviceAccountQuerier: scoped.NewUserDefinedServiceAccountQuerier(logger, clientFake),
-		catsrcQueueSet:         queueinformer.NewEmptyResourceQueueSet(),
+		catsrcQueueSet:        queueinformer.NewEmptyResourceQueueSet(),
 	}
 	op.sources = grpc.NewSourceStore(config.logger, 1*time.Second, 5*time.Second, op.syncSourceState)
 	if op.reconciler == nil {
@@ -1171,8 +1329,17 @@ func objectReference(name string) *corev1.ObjectReference {
 	return &corev1.ObjectReference{Name: name}
 }
 
-func toManifest(obj runtime.Object) string {
-	raw, _ := json.Marshal(obj)
+func yamlFromFilePath(t *testing.T, fileName string) string {
+	yaml, err := ioutil.ReadFile(fileName)
+	require.NoError(t, err)
+
+	return string(yaml)
+}
+
+func toManifest(t *testing.T, obj runtime.Object) string {
+	raw, err := json.Marshal(obj)
+	require.NoError(t, err)
+
 	return string(raw)
 }
 
@@ -1181,3 +1348,110 @@ func pod(s v1alpha1.CatalogSource) *corev1.Pod {
 	ownerutil.AddOwner(pod, &s, false, false)
 	return pod
 }
+
+func decodeFile(t *testing.T, file string, to runtime.Object) runtime.Object {
+	manifest := yamlFromFilePath(t, file)
+	dec := utilyaml.NewYAMLOrJSONDecoder(strings.NewReader(manifest), 10)
+	require.NoError(t, dec.Decode(to))
+
+	return to
+}
+
+type modifierFunc func(t *testing.T, obj runtime.Object) runtime.Object
+
+func modify(t *testing.T, obj runtime.Object, modifiers ...modifierFunc) runtime.Object {
+	o := obj.DeepCopyObject()
+	for _, modifier := range modifiers {
+		o = modifier(t, o)
+	}
+
+	return o
+}
+
+type metaModifierFunc func(m metav1.Object)
+
+func modifyMeta(mf metaModifierFunc) modifierFunc {
+	return func(t *testing.T, obj runtime.Object) runtime.Object {
+		accessor, err := meta.Accessor(obj)
+		require.NoError(t, err)
+
+		mf(accessor)
+
+		return obj
+	}
+}
+
+func withNamespace(namespace string) modifierFunc {
+	return modifyMeta(func(m metav1.Object) {
+		m.SetNamespace(namespace)
+	})
+}
+
+func withOwner(owner ownerutil.Owner) modifierFunc {
+	return modifyMeta(func(m metav1.Object) {
+		ownerutil.AddNonBlockingOwner(m, owner)
+	})
+}
+
+func withObjectMeta(t *testing.T, obj runtime.Object, m *metav1.ObjectMeta) runtime.Object {
+	o := obj.DeepCopyObject()
+	accessor, err := meta.Accessor(o)
+	require.NoError(t, err)
+
+	accessor.SetAnnotations(m.GetAnnotations())
+	accessor.SetClusterName(m.GetClusterName())
+	accessor.SetCreationTimestamp(m.GetCreationTimestamp())
+	accessor.SetDeletionGracePeriodSeconds(m.GetDeletionGracePeriodSeconds())
+	accessor.SetDeletionTimestamp(m.GetDeletionTimestamp())
+	accessor.SetFinalizers(m.GetFinalizers())
+	accessor.SetGenerateName(m.GetGenerateName())
+	accessor.SetGeneration(m.GetGeneration())
+	accessor.SetLabels(m.GetLabels())
+	accessor.SetManagedFields(m.GetManagedFields())
+	accessor.SetName(m.GetName())
+	accessor.SetNamespace(m.GetNamespace())
+	accessor.SetOwnerReferences(m.GetOwnerReferences())
+	accessor.SetResourceVersion(m.GetResourceVersion())
+	accessor.SetSelfLink(m.GetSelfLink())
+	accessor.SetUID(m.GetUID())
+
+	return o
+}
+
+func apiResourcesForObjects(objs []runtime.Object) []*metav1.APIResourceList {
+	apis := []*metav1.APIResourceList{}
+	for _, o := range objs {
+		switch o.(type) {
+		case *v1beta1.CustomResourceDefinition:
+			crd := o.(*v1beta1.CustomResourceDefinition)
+			apis = append(apis, &metav1.APIResourceList{
+				GroupVersion: metav1.GroupVersion{Group: crd.Spec.Group, Version: crd.Spec.Versions[0].Name}.String(),
+				APIResources: []metav1.APIResource{
+					{
+						Name:         crd.GetName(),
+						SingularName: crd.Spec.Names.Singular,
+						Namespaced:   crd.Spec.Scope == v1beta1.NamespaceScoped,
+						Group:        crd.Spec.Group,
+						Version:      crd.Spec.Versions[0].Name,
+						Kind:         crd.Spec.Names.Kind,
+					},
+				},
+			})
+		case *apiregistrationv1.APIService:
+			a := o.(*apiregistrationv1.APIService)
+			names := strings.Split(a.Name, ".")
+			apis = append(apis, &metav1.APIResourceList{
+				GroupVersion: metav1.GroupVersion{Group: names[1], Version: a.Spec.Version}.String(),
+				APIResources: []metav1.APIResource{
+					{
+						Name:    names[1],
+						Group:   names[1],
+						Version: a.Spec.Version,
+						Kind:    names[1] + "Kind",
+					},
+				},
+			})
+		}
+	}
+	return apis
+}
